package com.ruoyi.project.process.service.impl;

import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.project.process.mapper.TMocMapper;
import com.ruoyi.project.process.domain.TMoc;
import com.ruoyi.project.process.service.ITMocService;

/**
 * MOC管理Service业务层处理
 *
 * @author ruoyi
 * @date 2020-12-07
 */
@Service
public class TMocServiceImpl implements ITMocService
{
    @Autowired
    private TMocMapper tMocMapper;

    /**
     * 查询MOC管理
     *
     * @param id MOC管理ID
     * @return MOC管理
     */
    @Override
    public TMoc selectTMocById(Long id)
    {
        return tMocMapper.selectTMocById(id);
    }

    /**
     * 查询MOC管理列表
     *
     * @param tMoc MOC管理
     * @return MOC管理
     */
    @Override
    public List<TMoc> selectTMocList(TMoc tMoc)
    {
        return tMocMapper.selectTMocList(tMoc);
    }

    /**
     * 新增MOC管理
     *
     * @param tMoc MOC管理
     * @return 结果
     */
    @Override
    public int insertTMoc(TMoc tMoc)
    {
        return tMocMapper.insertTMoc(tMoc);
    }

    /**
     * 修改MOC管理
     *
     * @param tMoc MOC管理
     * @return 结果
     */
    @Override
    public int updateTMoc(TMoc tMoc)
    {
        return tMocMapper.updateTMoc(tMoc);
    }

    /**
     * 批量删除MOC管理
     *
     * @param ids 需要删除的MOC管理ID
     * @return 结果
     */
    @Override
    public int deleteTMocByIds(Long[] ids)
    {
        return tMocMapper.deleteTMocByIds(ids);
    }

    /**
     * 删除MOC管理信息
     *
     * @param id MOC管理ID
     * @return 结果
     */
    @Override
    public int deleteTMocById(Long id)
    {
        return tMocMapper.deleteTMocById(id);
    }
}
